
;; sid_player - Copyright 2008 by Michael Kohn
;; Email: mike@mikekohn.net
;;   Web: http://www.mikekohn.net/
;;
;; Circuit is a MOS 6581 SID chip (the sound chip from a Commodore 64)
;; connected to an ATmega168 connected over rs232 to a computer.
;; This software should allow the computer to control the SID chip through
;; the Ateml.

.include "m168def.inc"

; note: CLKSEL 0110
;       CKLOUT
;       CKDIV8

.dseg

.cseg
  ;; Prescale the clock.  SID wants 1MHz, so let's run everything
  ;; at 1MHz (from an 4MHz resonator since that's all I have)
  ldi r17, (1<<CLKPCE)
  sts CLKPR, r17
  ldi r17, (1<<CLKPS1)
  sts CLKPR, r17

  ;; Set up stack ptr
  ldi r16, RAMEND>>8
  out SPH, r16
  ldi r16, RAMEND&255
  out SPL, r16

  ;; Set up PORTB to output
  ser r17
  out DDRB, r17             ; entire port B is output
  eor r18, r18
  out PORTB, r18            ; turn off entire port

  ;; Set up PORTC to output
  ser r17
  out DDRC, r17             ; entire port C is output
  ldi r18, 32               ; /CS = 1, rest of port is 0
  out PORTC, r18            ; turn off entire port

  ;; Set up PORTD to output
  ser r17
  out DDRD, r17             ; entire port D is output
  eor r18, r18
  out PORTD, r18            ; turn off entire port

  ;; Set up rs232 baud rate
  eor r16, r16
  sts UBRR0H, r16
  ldi r16, 12               ; 25 @ 1MHz = 2400 baud
  ;ldi r16, 2               ; 25 @ 1MHz = 2400 baud
  sts UBRR0L, r16           ; 25 @ 8MHz = 19200 baud

  ;; Set up rs232 options
  ldi r16, (1<<UCSZ00)+(1<<UCSZ01)    ; sets up data as 8N1
  sts UCSR0C, r16
  ldi r17, (1<<TXEN0)+(1<<RXEN0)      ; enables send/receive
  sts UCSR0B, r17
  ;eor r17, r17
  ldi r17, (1<<U2X0)
  sts UCSR0A, r17

  ; call test_sid
  ldi r23, 0              ; state = 0

main_loop:
  lds r20, UCSR0A         ; poll uart to see if there is a data waiting
  sbrs r20, RXC0
  rjmp main_loop          ; if no data, loop around

  lds r17, UDR0           ; read char from uart

  tst r23
  brne not_state0
  cpi r17, 0xfe          ; copyright request
  brne not_copyright
  ldi r30, (copyright*2)&255
  ldi r31, (copyright*2)>>8
  call send_string
  rjmp main_loop
not_copyright:
  cpi r17, 0xff          ; state 0 is just waiting for 0xff
  brne main_loop
  inc r23
  rjmp main_loop

not_state0:
  cpi r23, 1             ; state 1 is waiting for an address
  brne not_state1
  inc r23
  mov r18, r17
  rjmp main_loop

not_state1:
  eor r23, r23            ; state 2 is waiting for a data byte
  call poke               ; you're all clear kid, now let's poke this thing
  rjmp main_loop          ; and go home!

  nop                     ; lazy bastard doing nothing
  sleep                   ; damn, i'm fucking tired

poke:   ; poke(r18=address, r17=data);  / r17,r18,r20
  ori r18, 32             ; set /CS off
  out PORTC, r18

  mov r20, r17            ; for port D
  andi r17, 0x1f
  lsl r17
  out PORTB, r17          ; output to port B ((r17&0x1f)<<1)
  andi r20, (128|64|32)
  out PORTD, r20

  andi r18, 0x1f          ; set /CS on
  out PORTC, r18
  ori r18, 32             ; set /CS off
  out PORTC, r18
  ret

;; send_char(r17);   / r14,r17
send_char:
  lds r14, UCSR0A     ; check to see if it's okay to send a char
  sbrs r14, UDRE0
  rjmp send_char      ; if it's not okay, loop around :)
  sts UDR0, r17       ; output a char over rs232
  ret

;; send_string(r31:r30);   / r17,r31,r30
send_string:
  lpm r17, Z+              ; read next char in string buffer (from flash)
  tst r17                  ; check for null termination
  breq exit_send_string
  rcall send_char
  rjmp send_string
exit_send_string:
  ret

;; test_sid()
test_sid:
  ;; Turn up volume
  ldi r18, 24             ; poke 54296, 15
  ldi r17, 15
  call poke

  ;; Voice 1: Set Attack/Decay | Sustain/Release
  ldi r18, 5              ; poke 54277, 190
  ldi r17, 190
  call poke
  inc r18                 ; poke 54278, 248
  ldi r17, 248
  call poke

  ;; Voice 1: Set Freq to A3
  eor r18, r18            ; poke 54272, 24
  ldi r17, 24
  call poke
  inc r18                 ; poke 54273, 14
  ldi r17, 14
  call poke

  ;; Voice 1: Set Waveform (Triangle)
  ldi r18, 4              ; poke 54276, 17
  ldi r17, 17
  call poke
  ret

copyright:
.db "sid_player for Atmel ATMega168\r\nCopyright 2008 - Michael Kohn\r\nVersion 0.12\r\n",0



